/** @file
 * $Source: /home/cvsmanager/yoda/techmodel/Calib1_Ac2Reader.cpp,v $
 * $Id: Calib1_Ac2Reader.cpp,v 6.1 2006/05/30 19:10:01 kusanagi Exp $
 * $Author: kusanagi $
 * 
 * Implementation of the Calib1_Ac2Reader class.
 */

#include <string>
#include <log4cxx/logger.h>
#include <fstream>
#include "stdio.h"
extern "C" {
    #include "CRC.h"
    #include "forroutines/anticounter/AC.h"
    extern int ACcalib(int length, unsigned char* calibpointer, struct calibstruct* calibpointer);
}

#include "ReaderAlgorithms.h"

using namespace pamela;
using namespace pamela::techmodel;

static log4cxx::LoggerPtr logger = log4cxx::Logger::getLogger(_T("pamela.techmodel.Calib1_Ac2Reader"));

/**
 * Constructor. 
 */
Calib1_Ac2Reader::Calib1_Ac2Reader(void): 
  TechmodelAlgorithm(PacketType::Calib1_Ac2, "Calib1_Ac2") { 
  logger->debug(_T("Constructor"));
  calib1_Ac2 = new Calib1_Ac2Event();
}

/**
 * Get a string with the version info of the algorithm.
 */
std::string Calib1_Ac2Reader::GetVersionInfo(void) const {
  return "$Header: /home/cvsmanager/yoda/techmodel/Calib1_Ac2Reader.cpp,v 6.1 2006/05/30 19:10:01 kusanagi Exp $\n";
}

/**
 * Initialize the algorithm with a special run. This will initialize the
 * event reader routines for all packet types.
 */
void Calib1_Ac2Reader::Init(PamelaRun *run) {
  logger->debug(_T("Initialize"));
  SetInputStream(run);
  run->WriteSubPacket(this, &calib1_Ac2, calib1_Ac2->Class());
}

/**
 * Unpack the Calib1_Ac2Reader event from an input file.
 */
void Calib1_Ac2Reader::RunEvent(int EventNumber, long int dataLength) throw (Exception){
    char   *subData = new char[dataLength];
    struct calibstruct output = {0};

    InputFile->read(subData, sizeof(unsigned char)*dataLength);

    calib1_Ac2->ERROR = ACcalib(dataLength, (unsigned char*)subData, &output);
    memcpy(calib1_Ac2->header,     output.header,     sizeof(output.header));
    memcpy(calib1_Ac2->status,     output.status,     sizeof(output.status));
    memcpy(calib1_Ac2->temp,       output.temp,       sizeof(output.temp));
    memcpy(calib1_Ac2->DAC1,       output.DAC1,       sizeof(output.DAC1));
    memcpy(calib1_Ac2->DAC2,       output.DAC2,       sizeof(output.DAC2));
    memcpy(calib1_Ac2->regist,     output.regist,     sizeof(output.regist));
    memcpy(calib1_Ac2->time,       output.time,       sizeof(output.time));
    calib1_Ac2->n_tr = output.n_tr;
    memcpy(calib1_Ac2->hitmap_tr,  output.hitmap_tr,  sizeof(output.hitmap_tr));
    memcpy(calib1_Ac2->curve1,     output.curve1,     sizeof(output.curve1));
    memcpy(calib1_Ac2->curve2,     output.curve2,     sizeof(output.curve2));
    calib1_Ac2->iCRC        = output.iCRC;
    calib1_Ac2->tail        = output.tail;
    calib1_Ac2->CRC         = output.CRC;
    calib1_Ac2->CRCcheck    = output.CRCcheck;

}

