/** @file
 * $Author: kusanagi $
 * $Date: 2006/02/07 10:43:06 $
 * $Revision: 5.3 $
 * 
 * Implementation of the TrkInitReader class.
 */

#include "ReaderAlgorithms.h"

using namespace pamela::techmodel;

static log4cxx::LoggerPtr logger = log4cxx::Logger::getLogger(_T("pamela.techmodel.TrkInitReader"));

/**
 * Constructor. 
 */
TrkInitReader::TrkInitReader(void): 
  TechmodelAlgorithm(PacketType::TrkInit, "TechmodelTrkInitReader") { 
  logger->debug(_T("Constructor"));
  trkInit = new TrkInitEvent();
}

/**
 * Get a string with the version info of the algorithm.
 */
std::string TrkInitReader::GetVersionInfo(void) const {
  return 
    "$Trailer: /home/cvsmanager/yoda/techmodel/TrkInitReader.cpp,v 1.1.1.1 2004/07/06 12:20:23 Maurizio Nagni Exp $\n";
}

/**
 * Initialize the algorithm with a special run. This will initialize the
 * event reader routines for all packet types.
 */
void TrkInitReader::Init(PamelaRun *run) {
  SetInputStream(run);
  run->WriteSubPacket(this, &trkInit, trkInit->Class());
}

/**
 * Unpack the TrkInit event from an input file.
 * The CPU does not add any CRC control at the packet end.
 * @param EventNumber 
 * @param dataLength 
 */
void TrkInitReader::RunEvent(int EventNumber, long int dataLength) throw (WrongCRCException){
    char      subData[dataLength];
    InputFile->read(subData, sizeof(subData));
    trkInit->trkInitData        = new TArrayC(dataLength, subData);
}

